// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: tensorflow/compiler/jit/xla_compilation_cache.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3009000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3009002 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_table_driven.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/inlined_string_field.h>
#include <google/protobuf/metadata.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "tensorflow/compiler/xla/service/hlo.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto {
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTableField entries[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::AuxillaryParseTableField aux[]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::ParseTable schema[2]
    PROTOBUF_SECTION_VARIABLE(protodesc_cold);
  static const ::PROTOBUF_NAMESPACE_ID::internal::FieldMetadata field_metadata[];
  static const ::PROTOBUF_NAMESPACE_ID::internal::SerializationTable serialization_table[];
  static const ::PROTOBUF_NAMESPACE_ID::uint32 offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto;
namespace tensorflow {
class XlaSerializedCacheEntry;
class XlaSerializedCacheEntryDefaultTypeInternal;
extern XlaSerializedCacheEntryDefaultTypeInternal _XlaSerializedCacheEntry_default_instance_;
class XlaSerializedCacheKey;
class XlaSerializedCacheKeyDefaultTypeInternal;
extern XlaSerializedCacheKeyDefaultTypeInternal _XlaSerializedCacheKey_default_instance_;
}  // namespace tensorflow
PROTOBUF_NAMESPACE_OPEN
template<> ::tensorflow::XlaSerializedCacheEntry* Arena::CreateMaybeMessage<::tensorflow::XlaSerializedCacheEntry>(Arena*);
template<> ::tensorflow::XlaSerializedCacheKey* Arena::CreateMaybeMessage<::tensorflow::XlaSerializedCacheKey>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace tensorflow {

// ===================================================================

class XlaSerializedCacheKey :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:tensorflow.XlaSerializedCacheKey) */ {
 public:
  XlaSerializedCacheKey();
  virtual ~XlaSerializedCacheKey();

  XlaSerializedCacheKey(const XlaSerializedCacheKey& from);
  XlaSerializedCacheKey(XlaSerializedCacheKey&& from) noexcept
    : XlaSerializedCacheKey() {
    *this = ::std::move(from);
  }

  inline XlaSerializedCacheKey& operator=(const XlaSerializedCacheKey& from) {
    CopyFrom(from);
    return *this;
  }
  inline XlaSerializedCacheKey& operator=(XlaSerializedCacheKey&& from) noexcept {
    if (GetArenaNoVirtual() == from.GetArenaNoVirtual()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const XlaSerializedCacheKey& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const XlaSerializedCacheKey* internal_default_instance() {
    return reinterpret_cast<const XlaSerializedCacheKey*>(
               &_XlaSerializedCacheKey_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(XlaSerializedCacheKey& a, XlaSerializedCacheKey& b) {
    a.Swap(&b);
  }
  inline void Swap(XlaSerializedCacheKey* other) {
    if (other == this) return;
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline XlaSerializedCacheKey* New() const final {
    return CreateMaybeMessage<XlaSerializedCacheKey>(nullptr);
  }

  XlaSerializedCacheKey* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<XlaSerializedCacheKey>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const XlaSerializedCacheKey& from);
  void MergeFrom(const XlaSerializedCacheKey& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  #if GOOGLE_PROTOBUF_ENABLE_EXPERIMENTAL_PARSER
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  #else
  bool MergePartialFromCodedStream(
      ::PROTOBUF_NAMESPACE_ID::io::CodedInputStream* input) final;
  #endif  // GOOGLE_PROTOBUF_ENABLE_EXPERIMENTAL_PARSER
  void SerializeWithCachedSizes(
      ::PROTOBUF_NAMESPACE_ID::io::CodedOutputStream* output) const final;
  ::PROTOBUF_NAMESPACE_ID::uint8* InternalSerializeWithCachedSizesToArray(
      ::PROTOBUF_NAMESPACE_ID::uint8* target) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(XlaSerializedCacheKey* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "tensorflow.XlaSerializedCacheKey";
  }
  private:
  inline ::PROTOBUF_NAMESPACE_ID::Arena* GetArenaNoVirtual() const {
    return nullptr;
  }
  inline void* MaybeArenaPtr() const {
    return nullptr;
  }
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto);
    return ::descriptor_table_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kDeviceTypeFieldNumber = 3,
    kPrefixFieldNumber = 4,
    kSignatureFingerprintFieldNumber = 1,
    kClusterFingerprintFieldNumber = 2,
  };
  // string device_type = 3;
  void clear_device_type();
  const std::string& device_type() const;
  void set_device_type(const std::string& value);
  void set_device_type(std::string&& value);
  void set_device_type(const char* value);
  void set_device_type(const char* value, size_t size);
  std::string* mutable_device_type();
  std::string* release_device_type();
  void set_allocated_device_type(std::string* device_type);

  // string prefix = 4;
  void clear_prefix();
  const std::string& prefix() const;
  void set_prefix(const std::string& value);
  void set_prefix(std::string&& value);
  void set_prefix(const char* value);
  void set_prefix(const char* value, size_t size);
  std::string* mutable_prefix();
  std::string* release_prefix();
  void set_allocated_prefix(std::string* prefix);

  // uint64 signature_fingerprint = 1;
  void clear_signature_fingerprint();
  ::PROTOBUF_NAMESPACE_ID::uint64 signature_fingerprint() const;
  void set_signature_fingerprint(::PROTOBUF_NAMESPACE_ID::uint64 value);

  // uint64 cluster_fingerprint = 2;
  void clear_cluster_fingerprint();
  ::PROTOBUF_NAMESPACE_ID::uint64 cluster_fingerprint() const;
  void set_cluster_fingerprint(::PROTOBUF_NAMESPACE_ID::uint64 value);

  // @@protoc_insertion_point(class_scope:tensorflow.XlaSerializedCacheKey)
 private:
  class _Internal;

  ::PROTOBUF_NAMESPACE_ID::internal::InternalMetadataWithArena _internal_metadata_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr device_type_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr prefix_;
  ::PROTOBUF_NAMESPACE_ID::uint64 signature_fingerprint_;
  ::PROTOBUF_NAMESPACE_ID::uint64 cluster_fingerprint_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  friend struct ::TableStruct_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto;
};
// -------------------------------------------------------------------

class XlaSerializedCacheEntry :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:tensorflow.XlaSerializedCacheEntry) */ {
 public:
  XlaSerializedCacheEntry();
  virtual ~XlaSerializedCacheEntry();

  XlaSerializedCacheEntry(const XlaSerializedCacheEntry& from);
  XlaSerializedCacheEntry(XlaSerializedCacheEntry&& from) noexcept
    : XlaSerializedCacheEntry() {
    *this = ::std::move(from);
  }

  inline XlaSerializedCacheEntry& operator=(const XlaSerializedCacheEntry& from) {
    CopyFrom(from);
    return *this;
  }
  inline XlaSerializedCacheEntry& operator=(XlaSerializedCacheEntry&& from) noexcept {
    if (GetArenaNoVirtual() == from.GetArenaNoVirtual()) {
      if (this != &from) InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return GetMetadataStatic().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return GetMetadataStatic().reflection;
  }
  static const XlaSerializedCacheEntry& default_instance();

  static void InitAsDefaultInstance();  // FOR INTERNAL USE ONLY
  static inline const XlaSerializedCacheEntry* internal_default_instance() {
    return reinterpret_cast<const XlaSerializedCacheEntry*>(
               &_XlaSerializedCacheEntry_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(XlaSerializedCacheEntry& a, XlaSerializedCacheEntry& b) {
    a.Swap(&b);
  }
  inline void Swap(XlaSerializedCacheEntry* other) {
    if (other == this) return;
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  inline XlaSerializedCacheEntry* New() const final {
    return CreateMaybeMessage<XlaSerializedCacheEntry>(nullptr);
  }

  XlaSerializedCacheEntry* New(::PROTOBUF_NAMESPACE_ID::Arena* arena) const final {
    return CreateMaybeMessage<XlaSerializedCacheEntry>(arena);
  }
  void CopyFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void MergeFrom(const ::PROTOBUF_NAMESPACE_ID::Message& from) final;
  void CopyFrom(const XlaSerializedCacheEntry& from);
  void MergeFrom(const XlaSerializedCacheEntry& from);
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  #if GOOGLE_PROTOBUF_ENABLE_EXPERIMENTAL_PARSER
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  #else
  bool MergePartialFromCodedStream(
      ::PROTOBUF_NAMESPACE_ID::io::CodedInputStream* input) final;
  #endif  // GOOGLE_PROTOBUF_ENABLE_EXPERIMENTAL_PARSER
  void SerializeWithCachedSizes(
      ::PROTOBUF_NAMESPACE_ID::io::CodedOutputStream* output) const final;
  ::PROTOBUF_NAMESPACE_ID::uint8* InternalSerializeWithCachedSizesToArray(
      ::PROTOBUF_NAMESPACE_ID::uint8* target) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  inline void SharedCtor();
  inline void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(XlaSerializedCacheEntry* other);
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "tensorflow.XlaSerializedCacheEntry";
  }
  private:
  inline ::PROTOBUF_NAMESPACE_ID::Arena* GetArenaNoVirtual() const {
    return nullptr;
  }
  inline void* MaybeArenaPtr() const {
    return nullptr;
  }
  public:

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  private:
  static ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadataStatic() {
    ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&::descriptor_table_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto);
    return ::descriptor_table_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto.file_level_metadata[kIndexInFileMessages];
  }

  public:

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kExecutableFieldNumber = 3,
    kKeyFieldNumber = 1,
    kHloModuleFieldNumber = 2,
  };
  // bytes executable = 3;
  void clear_executable();
  const std::string& executable() const;
  void set_executable(const std::string& value);
  void set_executable(std::string&& value);
  void set_executable(const char* value);
  void set_executable(const void* value, size_t size);
  std::string* mutable_executable();
  std::string* release_executable();
  void set_allocated_executable(std::string* executable);

  // .tensorflow.XlaSerializedCacheKey key = 1;
  bool has_key() const;
  void clear_key();
  const ::tensorflow::XlaSerializedCacheKey& key() const;
  ::tensorflow::XlaSerializedCacheKey* release_key();
  ::tensorflow::XlaSerializedCacheKey* mutable_key();
  void set_allocated_key(::tensorflow::XlaSerializedCacheKey* key);

  // .xla.HloModuleProto hlo_module = 2;
  bool has_hlo_module() const;
  void clear_hlo_module();
  const ::xla::HloModuleProto& hlo_module() const;
  ::xla::HloModuleProto* release_hlo_module();
  ::xla::HloModuleProto* mutable_hlo_module();
  void set_allocated_hlo_module(::xla::HloModuleProto* hlo_module);

  // @@protoc_insertion_point(class_scope:tensorflow.XlaSerializedCacheEntry)
 private:
  class _Internal;

  ::PROTOBUF_NAMESPACE_ID::internal::InternalMetadataWithArena _internal_metadata_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr executable_;
  ::tensorflow::XlaSerializedCacheKey* key_;
  ::xla::HloModuleProto* hlo_module_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  friend struct ::TableStruct_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// XlaSerializedCacheKey

// uint64 signature_fingerprint = 1;
inline void XlaSerializedCacheKey::clear_signature_fingerprint() {
  signature_fingerprint_ = PROTOBUF_ULONGLONG(0);
}
inline ::PROTOBUF_NAMESPACE_ID::uint64 XlaSerializedCacheKey::signature_fingerprint() const {
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheKey.signature_fingerprint)
  return signature_fingerprint_;
}
inline void XlaSerializedCacheKey::set_signature_fingerprint(::PROTOBUF_NAMESPACE_ID::uint64 value) {
  
  signature_fingerprint_ = value;
  // @@protoc_insertion_point(field_set:tensorflow.XlaSerializedCacheKey.signature_fingerprint)
}

// uint64 cluster_fingerprint = 2;
inline void XlaSerializedCacheKey::clear_cluster_fingerprint() {
  cluster_fingerprint_ = PROTOBUF_ULONGLONG(0);
}
inline ::PROTOBUF_NAMESPACE_ID::uint64 XlaSerializedCacheKey::cluster_fingerprint() const {
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheKey.cluster_fingerprint)
  return cluster_fingerprint_;
}
inline void XlaSerializedCacheKey::set_cluster_fingerprint(::PROTOBUF_NAMESPACE_ID::uint64 value) {
  
  cluster_fingerprint_ = value;
  // @@protoc_insertion_point(field_set:tensorflow.XlaSerializedCacheKey.cluster_fingerprint)
}

// string device_type = 3;
inline void XlaSerializedCacheKey::clear_device_type() {
  device_type_.ClearToEmptyNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline const std::string& XlaSerializedCacheKey::device_type() const {
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheKey.device_type)
  return device_type_.GetNoArena();
}
inline void XlaSerializedCacheKey::set_device_type(const std::string& value) {
  
  device_type_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), value);
  // @@protoc_insertion_point(field_set:tensorflow.XlaSerializedCacheKey.device_type)
}
inline void XlaSerializedCacheKey::set_device_type(std::string&& value) {
  
  device_type_.SetNoArena(
    &::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::move(value));
  // @@protoc_insertion_point(field_set_rvalue:tensorflow.XlaSerializedCacheKey.device_type)
}
inline void XlaSerializedCacheKey::set_device_type(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  
  device_type_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(value));
  // @@protoc_insertion_point(field_set_char:tensorflow.XlaSerializedCacheKey.device_type)
}
inline void XlaSerializedCacheKey::set_device_type(const char* value, size_t size) {
  
  device_type_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(),
      ::std::string(reinterpret_cast<const char*>(value), size));
  // @@protoc_insertion_point(field_set_pointer:tensorflow.XlaSerializedCacheKey.device_type)
}
inline std::string* XlaSerializedCacheKey::mutable_device_type() {
  
  // @@protoc_insertion_point(field_mutable:tensorflow.XlaSerializedCacheKey.device_type)
  return device_type_.MutableNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline std::string* XlaSerializedCacheKey::release_device_type() {
  // @@protoc_insertion_point(field_release:tensorflow.XlaSerializedCacheKey.device_type)
  
  return device_type_.ReleaseNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline void XlaSerializedCacheKey::set_allocated_device_type(std::string* device_type) {
  if (device_type != nullptr) {
    
  } else {
    
  }
  device_type_.SetAllocatedNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), device_type);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.XlaSerializedCacheKey.device_type)
}

// string prefix = 4;
inline void XlaSerializedCacheKey::clear_prefix() {
  prefix_.ClearToEmptyNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline const std::string& XlaSerializedCacheKey::prefix() const {
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheKey.prefix)
  return prefix_.GetNoArena();
}
inline void XlaSerializedCacheKey::set_prefix(const std::string& value) {
  
  prefix_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), value);
  // @@protoc_insertion_point(field_set:tensorflow.XlaSerializedCacheKey.prefix)
}
inline void XlaSerializedCacheKey::set_prefix(std::string&& value) {
  
  prefix_.SetNoArena(
    &::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::move(value));
  // @@protoc_insertion_point(field_set_rvalue:tensorflow.XlaSerializedCacheKey.prefix)
}
inline void XlaSerializedCacheKey::set_prefix(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  
  prefix_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(value));
  // @@protoc_insertion_point(field_set_char:tensorflow.XlaSerializedCacheKey.prefix)
}
inline void XlaSerializedCacheKey::set_prefix(const char* value, size_t size) {
  
  prefix_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(),
      ::std::string(reinterpret_cast<const char*>(value), size));
  // @@protoc_insertion_point(field_set_pointer:tensorflow.XlaSerializedCacheKey.prefix)
}
inline std::string* XlaSerializedCacheKey::mutable_prefix() {
  
  // @@protoc_insertion_point(field_mutable:tensorflow.XlaSerializedCacheKey.prefix)
  return prefix_.MutableNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline std::string* XlaSerializedCacheKey::release_prefix() {
  // @@protoc_insertion_point(field_release:tensorflow.XlaSerializedCacheKey.prefix)
  
  return prefix_.ReleaseNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline void XlaSerializedCacheKey::set_allocated_prefix(std::string* prefix) {
  if (prefix != nullptr) {
    
  } else {
    
  }
  prefix_.SetAllocatedNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), prefix);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.XlaSerializedCacheKey.prefix)
}

// -------------------------------------------------------------------

// XlaSerializedCacheEntry

// .tensorflow.XlaSerializedCacheKey key = 1;
inline bool XlaSerializedCacheEntry::has_key() const {
  return this != internal_default_instance() && key_ != nullptr;
}
inline void XlaSerializedCacheEntry::clear_key() {
  if (GetArenaNoVirtual() == nullptr && key_ != nullptr) {
    delete key_;
  }
  key_ = nullptr;
}
inline const ::tensorflow::XlaSerializedCacheKey& XlaSerializedCacheEntry::key() const {
  const ::tensorflow::XlaSerializedCacheKey* p = key_;
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheEntry.key)
  return p != nullptr ? *p : *reinterpret_cast<const ::tensorflow::XlaSerializedCacheKey*>(
      &::tensorflow::_XlaSerializedCacheKey_default_instance_);
}
inline ::tensorflow::XlaSerializedCacheKey* XlaSerializedCacheEntry::release_key() {
  // @@protoc_insertion_point(field_release:tensorflow.XlaSerializedCacheEntry.key)
  
  ::tensorflow::XlaSerializedCacheKey* temp = key_;
  key_ = nullptr;
  return temp;
}
inline ::tensorflow::XlaSerializedCacheKey* XlaSerializedCacheEntry::mutable_key() {
  
  if (key_ == nullptr) {
    auto* p = CreateMaybeMessage<::tensorflow::XlaSerializedCacheKey>(GetArenaNoVirtual());
    key_ = p;
  }
  // @@protoc_insertion_point(field_mutable:tensorflow.XlaSerializedCacheEntry.key)
  return key_;
}
inline void XlaSerializedCacheEntry::set_allocated_key(::tensorflow::XlaSerializedCacheKey* key) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaNoVirtual();
  if (message_arena == nullptr) {
    delete key_;
  }
  if (key) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena = nullptr;
    if (message_arena != submessage_arena) {
      key = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, key, submessage_arena);
    }
    
  } else {
    
  }
  key_ = key;
  // @@protoc_insertion_point(field_set_allocated:tensorflow.XlaSerializedCacheEntry.key)
}

// .xla.HloModuleProto hlo_module = 2;
inline bool XlaSerializedCacheEntry::has_hlo_module() const {
  return this != internal_default_instance() && hlo_module_ != nullptr;
}
inline const ::xla::HloModuleProto& XlaSerializedCacheEntry::hlo_module() const {
  const ::xla::HloModuleProto* p = hlo_module_;
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheEntry.hlo_module)
  return p != nullptr ? *p : *reinterpret_cast<const ::xla::HloModuleProto*>(
      &::xla::_HloModuleProto_default_instance_);
}
inline ::xla::HloModuleProto* XlaSerializedCacheEntry::release_hlo_module() {
  // @@protoc_insertion_point(field_release:tensorflow.XlaSerializedCacheEntry.hlo_module)
  
  ::xla::HloModuleProto* temp = hlo_module_;
  hlo_module_ = nullptr;
  return temp;
}
inline ::xla::HloModuleProto* XlaSerializedCacheEntry::mutable_hlo_module() {
  
  if (hlo_module_ == nullptr) {
    auto* p = CreateMaybeMessage<::xla::HloModuleProto>(GetArenaNoVirtual());
    hlo_module_ = p;
  }
  // @@protoc_insertion_point(field_mutable:tensorflow.XlaSerializedCacheEntry.hlo_module)
  return hlo_module_;
}
inline void XlaSerializedCacheEntry::set_allocated_hlo_module(::xla::HloModuleProto* hlo_module) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaNoVirtual();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(hlo_module_);
  }
  if (hlo_module) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(hlo_module)->GetArena();
    if (message_arena != submessage_arena) {
      hlo_module = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, hlo_module, submessage_arena);
    }
    
  } else {
    
  }
  hlo_module_ = hlo_module;
  // @@protoc_insertion_point(field_set_allocated:tensorflow.XlaSerializedCacheEntry.hlo_module)
}

// bytes executable = 3;
inline void XlaSerializedCacheEntry::clear_executable() {
  executable_.ClearToEmptyNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline const std::string& XlaSerializedCacheEntry::executable() const {
  // @@protoc_insertion_point(field_get:tensorflow.XlaSerializedCacheEntry.executable)
  return executable_.GetNoArena();
}
inline void XlaSerializedCacheEntry::set_executable(const std::string& value) {
  
  executable_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), value);
  // @@protoc_insertion_point(field_set:tensorflow.XlaSerializedCacheEntry.executable)
}
inline void XlaSerializedCacheEntry::set_executable(std::string&& value) {
  
  executable_.SetNoArena(
    &::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::move(value));
  // @@protoc_insertion_point(field_set_rvalue:tensorflow.XlaSerializedCacheEntry.executable)
}
inline void XlaSerializedCacheEntry::set_executable(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  
  executable_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), ::std::string(value));
  // @@protoc_insertion_point(field_set_char:tensorflow.XlaSerializedCacheEntry.executable)
}
inline void XlaSerializedCacheEntry::set_executable(const void* value, size_t size) {
  
  executable_.SetNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(),
      ::std::string(reinterpret_cast<const char*>(value), size));
  // @@protoc_insertion_point(field_set_pointer:tensorflow.XlaSerializedCacheEntry.executable)
}
inline std::string* XlaSerializedCacheEntry::mutable_executable() {
  
  // @@protoc_insertion_point(field_mutable:tensorflow.XlaSerializedCacheEntry.executable)
  return executable_.MutableNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline std::string* XlaSerializedCacheEntry::release_executable() {
  // @@protoc_insertion_point(field_release:tensorflow.XlaSerializedCacheEntry.executable)
  
  return executable_.ReleaseNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited());
}
inline void XlaSerializedCacheEntry::set_allocated_executable(std::string* executable) {
  if (executable != nullptr) {
    
  } else {
    
  }
  executable_.SetAllocatedNoArena(&::PROTOBUF_NAMESPACE_ID::internal::GetEmptyStringAlreadyInited(), executable);
  // @@protoc_insertion_point(field_set_allocated:tensorflow.XlaSerializedCacheEntry.executable)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace tensorflow

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_tensorflow_2fcompiler_2fjit_2fxla_5fcompilation_5fcache_2eproto
